#ifndef PARTICLE_H
#define PARTICLE_H


#define PARTICLE_WRAP           1
#define PARTICLE_ALIVE          2
#define PARTICLE_ALPHA_DIMINISH 4
#define PARTICLE_ALPHA          8
#define PARTICLE_ROTATES       16
#define PARTICLE_COLLIDE       32
#define PARTICLE_INVISIBLE     64
#define PARTICLE_OSCILLATE_V  128
#define PARTICLE_OSCILLATE_H  256
#define PARTICLE_LOOP_ANIM    512

struct PARTICLE;

typedef struct PARTICLE {
	float x, y;
	float vx, vy;

	int old_x, old_y;

	fixed angle;
	fixed rot_speed;
	fixed old_angle;

	BITMAP **bmp, *aux_bmp;
	int num_bitmaps;

	fixed cur_frame;
	fixed start_frame;
	fixed end_frame;
	fixed anim_speed;
	int last_frame;

	int alpha, alpha_speed;

	int prop, old_prop;
	int life;
	int type;
	fixed power;

	int osc_v_mag, osc_h_mag;
	fixed osc_v_phase, osc_v_speed;
	fixed osc_h_phase, osc_h_speed;
	fixed osc_v_angle;
	fixed osc_h_angle;

	int d1, d2;

	struct PARTICLE *owner;
} PARTICLE;

extern PARTICLE **Particle;
extern int NumParticles;


PARTICLE *create_particle();
void set_particle_coord(PARTICLE *p, float x, float y);
void set_particle_velocity(PARTICLE *p, float vx, float vy);
void set_particle_angle(PARTICLE *p, float angle);
void set_particle_rotation_speed(PARTICLE *p, fixed rot_speed);
void set_particle_alpha(PARTICLE *p, int a);
void set_particle_owner(PARTICLE *p, PARTICLE *owner);

void add_particle_bitmap(PARTICLE *p, BITMAP *bmp);
void remove_particle_bitmap(PARTICLE *p, BITMAP *bmp);
void remove_particle_bitmaps(PARTICLE *p);
int get_particle_bitmap_w(PARTICLE *p);
int get_particle_bitmap_h(PARTICLE *p);
struct BITMAP *get_particle_current_frame(PARTICLE *p);
struct BITMAP *get_particle_current_bitmap(PARTICLE *p);

void set_particle_property(PARTICLE *p, int val);
void get_particle_velocity(PARTICLE *p, float *vx, float *vy);
void get_particle_coord(PARTICLE *p, float *x, float *y);
void set_particle_life(PARTICLE *p, int val);
void set_particle_type(PARTICLE *p, int val);
extern void set_particle_power(PARTICLE *p, int power);

extern void set_particle_frame(PARTICLE *p, float frame);
extern void set_particle_frames(PARTICLE *p, float start, float end);
extern void set_particle_anim_speed(PARTICLE *p, float speed);


extern void set_particle_osc_v(PARTICLE *p, int mag, float speed);
extern void set_particle_osc_h(PARTICLE *p, int mag, float speed);


float get_particle_angle(PARTICLE *p);
void destroy_particle(PARTICLE *p);
void draw_particles(BITMAP *bmp);
void clear_particles();
int install_particle();

#endif
