/* asteroid.c - Asteroid managing module
 * 
 * Copyright 2001 by Robert J Ohannessian.
 *
 * This code is placed under the GNU General Public License.
 * Please refer to the accompanying file 'copying.txt' for details.
 */

#include <string.h>

#include "allegro.h"
#include "particle.h"
#include "asteroid.h"
#include "gfx.h"


#ifndef M_PI
#define M_PI 3.14159
#endif

ASTEROID *create_asteroid(int family, int size) {

	ASTEROID *a = malloc(sizeof(ASTEROID));
	memset(a, 0, sizeof(ASTEROID));

	a->Particle = create_particle();
	set_particle_property(a->Particle, PARTICLE_WRAP | PARTICLE_COLLIDE | PARTICLE_ROTATES);
	set_particle_coord(a->Particle, rand() % DISPLAY_W, rand() % DISPLAY_H);
	set_particle_velocity(a->Particle, (rand() % DISPLAY_W - DISPLAY_W / 2) / 4.0, (rand() % DISPLAY_H - DISPLAY_H / 2) / 4.0);
	set_particle_angle(a->Particle, (rand() % 360) / 180.0 * M_PI);
	set_particle_rotation_speed(a->Particle, itofix(rand() % 100) - itofix(50));

	a->family = MID(0, family, 3);
	a->size = MID(0, size, 3);
	a->hp = itofix(rand() % 8 + 10);
	if (size > 0)
		a->hp = itofix(1);
	a->maxhp = a->hp;
	a->Particle->type = 'A';
	
	return a;
}

void destroy_asteroid(ASTEROID *a) {

	if (!a)
		return;

	if (a->Particle)
		destroy_particle(a->Particle);

	free(a);

	return;
}
