/* dataload.c - Data loading module
 * 
 * Copyright 2001 by Robert J Ohannessian.
 *
 * This code is placed under the GNU General Public License.
 * Please refer to the accompanying file 'copying.txt' for details.
 */

#include "allegro.h"
#include "gfx.h"
#include "demo.h"
#include "jgmod.h"
#include "weapon.h"
#include "anim.h"
#include "game.h"


#ifndef DATADIR
#define DATADIR "data/"
#endif

int LoadData() {
	
	int load_w = 300, load_h = 20;
	int total_steps = 14, step = 0;
	int r, g, b;
	BITMAP *data, *temp[3];

	textout(screen, font, "Loading...", DISPLAY_W / 2 - load_w / 2, DISPLAY_H / 2 - load_h / 2 - 10, makecol(255, 255, 255));
	rect(screen, DISPLAY_W / 2 - load_w / 2, DISPLAY_H / 2 - load_h / 2, DISPLAY_W / 2 + load_w / 2, DISPLAY_H / 2 + load_h / 2, makecol(255, 0, 0));

#define UPDATE_LOAD_BAR(foo) { \
	hsv_to_rgb(step * 72.0 / total_steps, 1, 1, &r, &g, &b); \
	rectfill(screen, DISPLAY_W / 2 - load_w / 2 + 2 + step * (load_w - 3) / total_steps, DISPLAY_H / 2 - load_h / 2 + 2, \
		DISPLAY_W / 2 - load_w / 2 + (step + 1) * (load_w - 3) / total_steps, DISPLAY_H / 2 + load_h / 2 - 2, makecol(r, g, b)); \
		text_mode(0); \
		textout(screen, font, foo, DISPLAY_W / 2 - load_w / 2, DISPLAY_H / 2 + load_h / 2 + 2, makecol(255, 255, 255)); \
		step++; \
	}
	
	if (bitmap_color_depth(screen) != 8)
		set_color_conversion(COLORCONV_TOTAL);
	else
		set_color_conversion(COLORCONV_NONE);

	/* Load Demo data file, convert to color depth, scale if necessary */
	demodata = load_datafile(DATADIR "demo.dat");
	if (!demodata) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Can't load datafile demo.dat!\n");
		return -2;
	}


	GamePal = (RGB*)demodata[GAME_PAL].dat;	


	if (bitmap_color_depth(screen) == 8)
		set_palette(GamePal);
	
	textout(screen, font, "Loading...", DISPLAY_W / 2 - load_w / 2, DISPLAY_H / 2 - load_h / 2 - 10, makecol(255, 255, 255));
	rect(screen, DISPLAY_W / 2 - load_w / 2, DISPLAY_H / 2 - load_h / 2, DISPLAY_W / 2 + load_w / 2, DISPLAY_H / 2 + load_h / 2, makecol(255, 0, 0));

	UPDATE_LOAD_BAR("Color mapping tables     ");

	if (!exists(DATADIR "colormap.bin")) {
		PACKFILE *file;
		create_rgb_table(&palette_map, GamePal, NULL);

		rgb_map = &palette_map;
		rgb_map->data[31][0][31] = 0;

		set_add_blender(128, 128, 128, 255);
		create_blender_table(&(transmaparray[3]), GamePal, NULL);
	
		set_add_blender(128, 128, 128, 191);
		create_blender_table(&(transmaparray[2]), GamePal, NULL);

		set_add_blender(128, 128, 128, 127);
		create_blender_table(&(transmaparray[1]), GamePal, NULL);
	
		set_add_blender(128, 128, 128, 63);
		create_blender_table(&(transmaparray[0]), GamePal, NULL);

		set_add_blender(128, 128, 128, 255);

		file = pack_fopen(DATADIR "colormap.bin", "wp");

		if (file) {
			pack_fwrite(&palette_map, sizeof(RGB_MAP), file);
			pack_fwrite(&transmaparray[0], sizeof(COLOR_MAP), file);
			pack_fwrite(&transmaparray[1], sizeof(COLOR_MAP), file);
			pack_fwrite(&transmaparray[2], sizeof(COLOR_MAP), file);
			pack_fwrite(&transmaparray[3], sizeof(COLOR_MAP), file);
			pack_fclose(file);
		}
	}
	else {
		PACKFILE *file;

		file = pack_fopen(DATADIR "colormap.bin", "rp");

		if (file) {
			pack_fread(&palette_map, sizeof(RGB_MAP), file);
			pack_fread(&transmaparray[0], sizeof(COLOR_MAP), file);
			pack_fread(&transmaparray[1], sizeof(COLOR_MAP), file);
			pack_fread(&transmaparray[2], sizeof(COLOR_MAP), file);
			pack_fread(&transmaparray[3], sizeof(COLOR_MAP), file);
			pack_fclose(file);
		}
		rgb_map = &palette_map;
	}
	color_map = &transmaparray[3];


	UPDATE_LOAD_BAR("Asteroid A               ");

	temp[0] =  demodata[ASTA_BMP].dat;
	data = create_bitmap(temp[0]->w * 2, temp[0]->h * 2);
	stretch_blit(temp[0], data, 0, 0, temp[0]->w, temp[0]->h, 0, 0, data->w, data->h);
	temp[1] = create_bitmap(temp[0]->w / 2, temp[0]->h / 2);
	stretch_blit(temp[0], temp[1], 0, 0, temp[0]->w, temp[0]->h, 0, 0, temp[1]->w, temp[1]->h);

	Ast[0] = data;
	Ast[1] = temp[0];
	Ast[2] = temp[1];

	UPDATE_LOAD_BAR("Asteroid B               ");

	temp[0] =  demodata[ASTE_BMP].dat;
	data = create_bitmap(temp[0]->w / 2, temp[0]->h / 2);
	stretch_blit(temp[0], data, 0, 0, temp[0]->w, temp[0]->h, 0, 0, data->w, data->h);
	temp[1] = create_bitmap(temp[0]->w / 4, temp[0]->h / 4);
	stretch_blit(temp[0], temp[1], 0, 0, temp[0]->w, temp[0]->h, 0, 0, temp[1]->w, temp[1]->h);

	Ast[3] = temp[0];
	Ast[4] = data;
	Ast[5] = temp[1];


	UPDATE_LOAD_BAR("Asteroid C               ");
	
	temp[0] =  demodata[ASTD_BMP].dat;
	data = create_bitmap(temp[0]->w / 2, temp[0]->h / 2);
	stretch_blit(temp[0], data, 0, 0, temp[0]->w, temp[0]->h, 0, 0, data->w, data->h);
	temp[1] = create_bitmap(temp[0]->w / 4, temp[0]->h / 4);
	stretch_blit(temp[0], temp[1], 0, 0, temp[0]->w, temp[0]->h, 0, 0, temp[1]->w, temp[1]->h);

	Ast[6] = temp[0];
	Ast[7] = data;
	Ast[8] = temp[1];

	UPDATE_LOAD_BAR("Ship                     ");

	data = demodata[SHIP_BMP].dat;
	PlayerBitmap[0] = data;
	data = demodata[SHIP2_BMP].dat;
	PlayerBitmap[1] = data;

	UPDATE_LOAD_BAR("Sound samples            ");
	
	Boom = (SAMPLE*)demodata[BOOM_WAV].dat;
	Tick = (SAMPLE*)demodata[TICK_WAV].dat;
	Engine = (SAMPLE*)demodata[ENGINELOOP_WAV].dat;
	PowerUp = (SAMPLE*)demodata[POWERUP_WAV].dat;

	UPDATE_LOAD_BAR("Font                     ");

	DigitFont = (FONT*)demodata[DIGIT_FNT].dat;
		
	UPDATE_LOAD_BAR("Misc images              ");

	Smoke =  demodata[SMOKE_BMP].dat;
	Bullet = demodata[BULLET_BMP].dat;
	Dust =   demodata[DUST_BMP].dat;

	UPDATE_LOAD_BAR("Music                    ");

	Music = load_mod(DATADIR "demo.dat#ZASX_XM");

	UPDATE_LOAD_BAR("Weapons                  ");

	NormalGun = load_weapon(DATADIR "demo.dat#NORMALGUN_WPN");
	RapidFireGun = load_weapon(DATADIR "demo.dat#RAPIDFIRE_WPN");
	LaserGun =  load_weapon(DATADIR "demo.dat#LASER_WPN");	
	Laser2Gun = load_weapon(DATADIR "demo.dat#LASER2_WPN");	
	Laser3Gun = load_weapon(DATADIR "demo.dat#LASER3_WPN");	

	UPDATE_LOAD_BAR("Health power-up          ");	
	load_anim_bank("Health", DATADIR "health.dat");
	UPDATE_LOAD_BAR("RapidFire power-up       ");
	load_anim_bank("RapidFire", DATADIR "rapid.dat");
	UPDATE_LOAD_BAR("Laser power-up           ");
	load_anim_bank("LaserGun", DATADIR "laser.dat");
	UPDATE_LOAD_BAR("Done.                    ");

#undef UPDATE_LOAD_BAR

	return 0;
}

int UnloadData() {

	destroy_weapon(NormalGun);
	destroy_weapon(RapidFireGun);
	destroy_weapon(LaserGun);
	destroy_weapon(Laser2Gun);
	destroy_weapon(Laser3Gun);

	unload_datafile(demodata);
	remove_anim();

	return 0;
}
