/* gamemenu.c - Menu manager module
 * 
 * Copyright 2001 by Robert J Ohannessian.
 *
 * This code is placed under the GNU General Public License.
 * Please refer to the accompanying file 'copying.txt' for details.
 */

#include <math.h>

#include "allegro.h"
#include "game.h"

#include "input.h"
#include "config.h"
#include "gfx.h"
#include "log.h"
#include "scorer.h"
#include "gametimer.h"

#ifndef M_PI
#define M_PI 3.14159
#endif


int ZasxMenu(int msg, DIALOG *d, int c) {

	int i;
	BITMAP *dest = (BITMAP*)d->dp3;
	FONT *f = (FONT*)d->dp2;
	char **menu = (char**)d->dp;
	int numitems = d->d1;
	int sel = d->d2;

	int dimmed;

	if (!d)
		return D_O_K;

	switch (msg) {

		case MSG_START:
			return D_WANTFOCUS;


		case MSG_DRAW:

			if (!dest || !f || !menu)
				return D_O_K;

			dimmed = makecol(getr(d->fg) / 2, getg(d->fg) / 2, getb(d->fg) / 2);

			rect(dest, d->x, d->y, d->x + d->w - 1, d->y + d->h - 1, dimmed);

			text_mode(d->bg);
			textout(dest, f, menu[0], d->x + 5, d->y + 5, d->fg);
			hline(dest, d->x + 5, d->y + text_height(f) + 7, d->x + d->w - 5, d->fg);
			hline(dest, d->x + 5, d->y + text_height(f) + 8, d->x + d->w - 5, dimmed);

			for (i = 1; i < numitems; i++) {
				if (i == sel) {
					rect(dest, d->x + 5, d->y + (text_height(f) + 4) * i + 16, d->x + d->w - 6, d->y + (text_height(f) + 4) * (i + 1) + 15, makecol(255, 0, 0));
					rect(dest, d->x + 6, d->y + (text_height(f) + 4) * i + 17, d->x + d->w - 7, d->y + (text_height(f) + 4) * (i + 1) + 14, makecol(255, 255, 0));
				}				
				if (menu[i])
					textout_centre(dest, f, menu[i], d->x + d->w / 2, d->y + (text_height(f) + 4) * i + 21, d->fg);
			}
			break;

		case MSG_CHAR:
			if (((c >> 8) == KEY_UP) && (d->d2 > 1)) {
				d->d2--;
				play_sample(Tick, 255, 128, 1000, FALSE);
				return D_USED_CHAR;
			}
			else if (((c >> 8) == KEY_DOWN) && (d->d2 < numitems - 1)) {
				d->d2++;
				play_sample(Tick, 255, 128, 1000, FALSE);
				return D_USED_CHAR;
			}
			else if (((c >> 8) == KEY_ENTER) || ((c >> 8) == KEY_ENTER_PAD) || ((c >> 8) == KEY_SPACE))
				return D_CLOSE;

		case MSG_IDLE:
			return D_REDRAWME;

		case MSG_WANTFOCUS:
		case MSG_LOSTFOCUS:
		case MSG_KEY:
			return D_WANTFOCUS;
	}

	return D_O_K;
}


				


void DoMenu() {

	char *MainMenuItems[] = { "Main Menu", "Single Player", "Two Players", "Configure", "View High Score", "Credits", "Quit"};
	char *ConfigureItems[] = { "Configure", "Player 1 Controls", "Player 2 Controls", "Back"};
	char *ControlItems[9];
	char *ConfigKeyName[9] = {
		"Player %i Controls",
		"Input:      %14s",
		"Accelerate: %14s",
		"Slow Down:  %14s",
		"Turn Left:  %14s",
		"Turn Right: %14s",
		"Fire:       %14s",
		"Default",
		"Back"
	};
	char *KeyItems[1];
	int KeyConfig[] = {
		INPUT_KEY_UP,
		INPUT_KEY_DOWN,
		INPUT_KEY_LEFT,
		INPUT_KEY_RIGHT,
		INPUT_KEY_FIRE
	};

	
	char *str = "Zasx";
	int i, j;
	int CurMenu = 0;
	int LastEntry = 1;	

	int ConfigPlayer = 0;
	INPUT *ConfigDevice = NULL;
	int ConfigKey = 0, ConfigKey2 = 0, ConfigKey3 = 1;
	int ConfigJoystick = 0, ConfigAxis = 0, ConfigStick = 0, ConfigButton = 0;

	struct MINI_PART {
		float x, y;
		float o;
		float angle, oangle;
		int color;
	} p[100];

	BITMAP *text = create_bitmap(8*4, 8);

	DIALOG ZasxMainMenu[] = {
		/* proc,                       x,                y,   w,   h,                  fg,    bg,  key,   flags, d1, d2,  dp,         dp2,  dp3 */
		{ZasxMenu,     DISPLAY_W/2 - 170, DISPLAY_H/2 - 40, 340, 260,makecol(255,255,255),    -1,    0,       0,  7, LastEntry, MainMenuItems, DigitFont, BackBuffer },
		{NULL}
	};

	DIALOG ZasxConfigureMenu[] = {
		/* proc,                       x,                y,   w,   h,                  fg,    bg,  key,   flags, d1, d2,  dp,         dp2,  dp3 */
		{ZasxMenu,     DISPLAY_W/2 - 170, DISPLAY_H/2 - 30, 340, 230,makecol(255,255,255),    -1,    0,       0,  4,  1, ConfigureItems, DigitFont, BackBuffer },
		{NULL}
	};

	DIALOG ZasxControlMenu[] = {
		/* proc,                       x,                 y,   w,   h,                  fg,    bg,  key,   flags, d1, d2,  dp,         dp2,  dp3 */
		{ZasxMenu,     DISPLAY_W/2 - 260, DISPLAY_H/2 - 105, 520, 325,makecol(255,255,255),    -1,    0,       0,  9,  ConfigKey3, ControlItems, DigitFont, BackBuffer },
		{NULL}
	};

	DIALOG ZasxKeyMenu[] = {
		/* proc,                       x,                y,   w,   h,                  fg,    bg,  key,   flags, d1, d2,  dp,         dp2,  dp3 */
		{ZasxMenu,     DISPLAY_W/2 - 300, DISPLAY_H/2,      600,  60,makecol(255,255,255),    -1,    0,       0,  1,  1, KeyItems, DigitFont, BackBuffer },
		{NULL}
	};

	if (!text)
		return;


	ControlItems[0] = malloc(sizeof(char) * 350);

	if (!ControlItems[0]) {
		destroy_bitmap(text);
		return;
	}

	KeyItems[0] = malloc(sizeof(char) * 60);

	if (!KeyItems[0]) {
		free(ControlItems[0]);
		destroy_bitmap(text);
		return;
	}


	for (i = 1; i < 9; i++)
		ControlItems[i] = ControlItems[0] + 35 * i;

	clear(text);
	text_mode(-1);
	textout(text, font, str, 0, 0, makecol(255, 255, 255));

	/* Stars */
	for (i = 0; i < 100; i++) {			
		p[i].o = rand() % DISPLAY_W - DISPLAY_W / 2;
		p[i].oangle = (rand()%360) * M_PI / 180;
		p[i].color = makecol(192, 192, 192);
	}

	do {
		
		DIALOG *ZasxDialog = NULL;
		DIALOG_PLAYER *ZasxDialogPlayer = NULL;
	
		if (CurMenu == 0)
			ZasxDialog = ZasxMainMenu;
		else if (CurMenu == 1)
			ZasxDialog = ZasxConfigureMenu;
		else if (CurMenu == 2)
			ZasxDialog = ZasxControlMenu;
		else if (CurMenu == 3)
			ZasxDialog = ZasxKeyMenu;

		ZasxDialogPlayer = init_dialog(ZasxDialog, 0);

		clear_keybuf();		

		do {
			GTIME_Poll();
	
			clear(BackBuffer);			

			for (i = 0; i < 100; i++) {
				p[i].x = p[i].o * cos(p[i].angle) + DISPLAY_W / 2;
				p[i].y = p[i].o * sin(p[i].angle) + DISPLAY_H / 2;
				p[i].angle = p[i].oangle + GameTime / 2000.0;

				putpixel(BackBuffer, (int)p[i].x, (int)p[i].y, p[i].color);
			}

			for (i = 0; i < text->h; i++) {
				for (j = 0; j < text->w; j++) {
					if (getpixel(text, j, i)) {
						draw_trans_sprite(BackBuffer, Bullet, j * Bullet->w + DISPLAY_W / 2 - text->w * Bullet->w / 2, i * Bullet->h + 10);
						draw_trans_sprite(BackBuffer, Bullet, j * Bullet->w + DISPLAY_W / 2 - text->w * Bullet->w / 2 + (int)(sin(GameTime / 600.0 + i * M_PI / 12) * DISPLAY_W / 60), i * Bullet->h + 10);
					}
				}
			}

			textout_right(BackBuffer, font, "Zasx  v" VERSION " - SpeedHack 2001", DISPLAY_W - 1, DISPLAY_H - text_height(font) * 2 - 1, makecol(255, 255, 255));
			textout_right(BackBuffer, font, "By Robert J Ohannessian", DISPLAY_W - 1, DISPLAY_H - text_height(font) - 1, makecol(255, 255, 255));

			//poll_joystick();
			/* Grab input before GUI */
			if (CurMenu == 3) {
				if (get_input_type(ConfigDevice) == INPUT_TYPE_KEYBOARD) {
					for (i = 1; i < KEY_MAX; i++) {
						if (key[i]) {
							ConfigKey2 = i;
							i = 0;
							break;
						}
					}
					if (!i)
						break;
				}
				else if (get_input_type(ConfigDevice) == INPUT_TYPE_JOYSTICK) {
					int found = 0;
					for (i = 0; i < joy[ConfigJoystick].num_sticks; i++) {
						int j;
						for (j = 0; j < joy[ConfigJoystick].stick[i].num_axis; j++) {
							if (joy[ConfigJoystick].stick[i].axis[j].d2) {
								ConfigAxis = j;
								ConfigStick = i;
								found = 1;
								break;
							}
							if (joy[ConfigJoystick].stick[i].axis[j].d1) {
								ConfigAxis = -j - 1;
								ConfigStick = i;
								found = 1;
								break;
							}
						}
						if (found)
							break;
					}
					if (found)
						break;

					for (i = 0; i < joy[ConfigJoystick].num_buttons; i++) {
						if (joy[ConfigJoystick].button[i].b) {
							ConfigButton = i;
							found = 1;
							break;
						}
					}
					if (found)
						break;
				}
			}			

			i = update_dialog(ZasxDialogPlayer);

			if (!i)
				break;

			SEND_MESSAGE(&ZasxDialog[0], MSG_DRAW, 0);

			blit(BackBuffer, screen, 0, 0, 0, 0, DISPLAY_W, DISPLAY_H);

		} while (1);

		play_sample(Boom, 128, 128, 2500, FALSE);

		LOG("Got out of main loop\n");
		LOG("Shutting down dialog\n");
		shutdown_dialog(ZasxDialogPlayer);
		LOG("Changing menu.\n");
	
		if (key[KEY_ESC])
			ZasxDialog[0].d2 = ZasxDialog[0].d1 - 1;
	
		i = ZasxDialog[0].d2;

		if (CurMenu == 0) {
			LastEntry = i;

			/* Exit */
			if (i == 6) {
				play_sample(Tick, 255, 128, 1000, FALSE);
				break;
			}
			/* Credits */
			else if (i == 5) {
				ShowCredits();
				play_sample(Tick, 255, 128, 1000, FALSE);
			}
			/* View score */
			else if (i == 4) {
				ViewHighScore();
				play_sample(Tick, 255, 128, 1000, FALSE);
			}
			/* Configure */
			else if (i == 3) {
				CurMenu = 1;
				continue;
			}
			/* Single player */
			else if (i == 1) {
				DoGame(1);
			}
			/* Two player */
			else if (i == 2) {
				DoGame(2);
			}
		}
		else if (CurMenu == 1) {
			if (i == 3) {
				CurMenu = 0;
				continue;
			}
			else if (i > 0) {
				char buf[15];
				INPUT *dev = load_player_input_config(i);

				if (!dev)
					continue;

				ConfigPlayer = i;
				ConfigDevice = dev;

				/* Set up menu for player config */
				sprintf(ControlItems[0], ConfigKeyName[0], i);				
				sprintf(ControlItems[1], ConfigKeyName[1], get_input_type_full_name(dev));

				for (i = 2; i < 7; i++)
					sprintf(ControlItems[i], ConfigKeyName[i], get_input_name(dev, KeyConfig[i - 2], buf, 14));

				sprintf(ControlItems[7], "%s", ConfigKeyName[7]);
				sprintf(ControlItems[8], "%s", ConfigKeyName[8]);
				
				CurMenu = 2;
			}
		}
		else if (CurMenu == 2) {

			if (i == 8) {
				save_player_config(ConfigPlayer, ConfigDevice);
				CurMenu = 1;
				continue;
			}
			/* Default */
			else if (i == 7) {
				char buf[15];
				INPUT *dev = ConfigDevice;
				get_default_player_config(ConfigPlayer, ConfigDevice);

				sprintf(ControlItems[0], ConfigKeyName[0], ConfigPlayer);				
				sprintf(ControlItems[1], ConfigKeyName[1], get_input_type_full_name(dev));

				for (i = 2; i < 7; i++)
					sprintf(ControlItems[i], ConfigKeyName[i], get_input_name(dev, KeyConfig[i - 2], buf, 14));

				sprintf(ControlItems[7], "%s", ConfigKeyName[7]);
				sprintf(ControlItems[8], "%s", ConfigKeyName[8]);

				CurMenu = 2;

				continue;
			}
			else if ((i >= 2) && (get_input_type(ConfigDevice) == INPUT_TYPE_KEYBOARD)) {
				ConfigKey = KeyConfig[i - 2];
				ConfigKey2 = get_input_key(ConfigDevice, ConfigKey);
				ConfigKey3 = i;
				CurMenu = 3;
				ControlItems[i][11] = 0;
				sprintf(KeyItems[0], "Enter key for %s", ControlItems[i]);
				ControlItems[i][11] = ' ';

				do {
					int x = -1;					
					for (j = 0; j < KEY_MAX; j++)
						if (key[j])
							x = j;

					if (x == -1)
						break;
				} while (1);

				while (keypressed()) readkey();
				clear_keybuf();

				continue;
			}
			else if ((i >= 2) && (get_input_type(ConfigDevice) == INPUT_TYPE_JOYSTICK)) {
				ConfigKey = KeyConfig[i - 2];				
				ConfigButton = get_input_joy_button(ConfigDevice, ConfigKey);
				ConfigStick  = get_input_joy_stick(ConfigDevice, ConfigKey);
				ConfigAxis   = get_input_joy_axis(ConfigDevice, ConfigKey);
				ConfigKey3 = i;
				CurMenu = 3;
				ControlItems[i][11] = 0;
				sprintf(KeyItems[0], "Press a button for %s", ControlItems[i]);
				ControlItems[i][11] = ' ';

				while (joy[0].button[0].b) poll_joystick();

				continue;
			}
			else if (i == 1) { /* Switch intput device */
				
				char buf[16];

				/* If keyboard, go to joystick */
				if (get_input_type(ConfigDevice) == INPUT_TYPE_KEYBOARD) {

					if (num_joysticks > 0) {
						destroy_input(&ConfigDevice);
						ConfigDevice = create_input(INPUT_TYPE_JOYSTICK);
						set_input(ConfigDevice, 0, "Joy0 0 0");						
						ConfigJoystick = 0;
						get_default_player_config(ConfigPlayer, ConfigDevice);
					}
				}
				/* If it's a joystick, go to next stick */
				else if (get_input_type(ConfigDevice) == INPUT_TYPE_JOYSTICK) {
					/* Is it the last stick? */					
					int n;
					get_input_name(ConfigDevice, 0, buf, 11);
					sscanf(buf, "Joy%i", &n);

					if ((n + 1) >= num_joysticks) {
						/* Then switch back to keyboard */
						destroy_input(&ConfigDevice);
						ConfigDevice = create_input(INPUT_TYPE_KEYBOARD);											
					}
					else {
						/* Go to next joystick */
						sprintf(buf, "Joy%i 0 0", n + 1);
						set_input(ConfigDevice, 0, buf);
						ConfigJoystick = n;						
					}
					get_default_player_config(ConfigPlayer, ConfigDevice);
				}				

				sprintf(ControlItems[0], ConfigKeyName[0], ConfigPlayer);				
				sprintf(ControlItems[1], ConfigKeyName[1], get_input_type_full_name(ConfigDevice));

				for (i = 2; i < 7; i++)
					sprintf(ControlItems[i], ConfigKeyName[i], get_input_name(ConfigDevice, KeyConfig[i - 2], buf, 14));

				sprintf(ControlItems[7], "%s", ConfigKeyName[7]);
				sprintf(ControlItems[8], "%s", ConfigKeyName[8]);
			}
		}
		else if (CurMenu == 3) {
			char buf[20];
			if (get_input_type(ConfigDevice) == INPUT_TYPE_KEYBOARD) {
				set_input_key(ConfigDevice, ConfigKey, ConfigKey2);
				sprintf(ControlItems[ConfigKey3], ConfigKeyName[ConfigKey3], get_input_name(ConfigDevice, ConfigKey, buf, 14));
			}
			else if (get_input_type(ConfigDevice) == INPUT_TYPE_JOYSTICK) {
				sprintf(buf, "Joy%i %i %i", ConfigJoystick, ConfigStick | ConfigButton, ConfigAxis);
				LOG("Setting input %s to %s\n", ConfigKeyName[ConfigKey3], buf);
				set_input(ConfigDevice, ConfigKey, buf);
				sprintf(ControlItems[ConfigKey3], ConfigKeyName[ConfigKey3], get_input_name(ConfigDevice, ConfigKey, buf, 14));				
			}
			CurMenu = 2;
			continue;
		}		
	} while (1);

   	destroy_bitmap(text);
	free(ControlItems[0]);
	free(KeyItems[0]);
	
	return;
}

